const fs = require('fs-extra');
const { exec } = require('child-process-promise');
const prompts = require('../src/prompts');
const collectAnswers = require('../src/collect-answers');

const emitInput = (ui, input) => {
  ui.process.subscribe(() => {
    setTimeout(() => {
      ui.rl.emit('line', input);
    }, 100);
  });
  ui.rl.emit('line', input);
};
const settingOfDefaultValue = new Promise(resolve => setTimeout(resolve, 10));
const tempFolder = 'temp-test-folder';
const createTestCwd = () => {
  beforeEach(async () => {
    await fs.ensureDir(tempFolder);
    process.chdir(tempFolder);
    expect(process.cwd()).toContain(tempFolder);
  });

  afterEach(async () => {
    process.chdir('..');
    await fs.remove(tempFolder).catch(err => {
      throw err;
    });
    expect(tempFolder).not.toBeExistingPath();
  });
};

describe('collect-answers', () => {
  it('asks all the questions', async () => {
    jest.spyOn(process.stdout, 'write');
    const collector = collectAnswers();

    prompts.forEach(async () => {
      await settingOfDefaultValue;
      emitInput(collector.ui);
    });

    await collector;
    prompts.forEach(prompt => {
      expect(process.stdout.write).toBeCalledWithArgumentContaining(
        prompt.message,
      );
    });
  });

  describe('projectName', () => {
    const prompt = [prompts.find(({ name }) => name === 'projectName')];
    createTestCwd();

    it('defaults to deslugified folder name', async () => {
      const collector = collectAnswers(prompt);
      await settingOfDefaultValue;
      await settingOfDefaultValue;
      emitInput(collector.ui);

      const answers = await collector;

      expect(answers.projectName).toEqual('Temp Test Folder');
    });

    it('returns answer we put in', async () => {
      const collector = collectAnswers(prompt);
      await settingOfDefaultValue;
      await settingOfDefaultValue;
      emitInput(collector.ui, 'Custom Answer');

      const answers = await collector;

      expect(answers.projectName).toEqual('Custom Answer');
    });
  });

  describe('projectNameSlug', () => {
    const prompt = [
      prompts.find(({ name }) => name === 'projectName'),
      prompts.find(({ name }) => name === 'projectNameSlug'),
    ];
    createTestCwd();

    it('defaults to slugified projectName', async () => {
      const collector = collectAnswers(prompt);
      await settingOfDefaultValue;
      emitInput(collector.ui);
      await settingOfDefaultValue;
      emitInput(collector.ui);

      const answers = await collector;

      expect(answers.projectNameSlug).toEqual('temp-test-folder');
    });

    it('returns answer we put in', async () => {
      const collector = collectAnswers(prompt);
      await settingOfDefaultValue;
      emitInput(collector.ui, 'Custom Answer');
      await settingOfDefaultValue;
      emitInput(collector.ui, 'custom-answer');

      const answers = await collector;

      expect(answers.projectNameSlug).toEqual('custom-answer');
    });
  });

  describe('projectAuthor', () => {
    const prompt = [prompts.find(({ name }) => name === 'projectAuthor')];
    let originalName;
    let originalEmail;

    beforeAll(async () => {
      const { stdout: name } = await exec('git config user.name');
      originalName = name.trim();
      const { stdout: email } = await exec('git config user.email');
      originalEmail = email.trim();

      await exec('git config user.name "John Doe"');
      await exec('git config user.email john.doe@domain.com');
    });

    afterAll(async () => {
      await exec(`git config user.name "${originalName}"`);
      await exec(`git config user.email "${originalEmail}"`);
    });

    it('defaults to git user', async () => {
      const collector = collectAnswers(prompt);
      await new Promise(resolve => setTimeout(resolve, 200));
      emitInput(collector.ui);

      const answers = await collector;

      expect(answers.projectAuthor).toEqual('John Doe<john.doe@domain.com>');
    });

    it('returns answer we put in', async () => {
      const collector = collectAnswers(prompt);
      await new Promise(resolve => setTimeout(resolve, 500));
      emitInput(collector.ui, 'Foo Bar<foo.bar@domain.com>');

      const answers = await collector;

      expect(answers.projectAuthor).toEqual('Foo Bar<foo.bar@domain.com>');
    });

    it('errors if author is provided without email', async () => {
      jest.spyOn(process.stdout, 'write');
      const collector = collectAnswers(prompt);
      await new Promise(resolve => setTimeout(resolve, 500));
      emitInput(collector.ui, 'wrong format');
      await new Promise(resolve => setTimeout(resolve, 500));
      emitInput(collector.ui, 'Foo Bar<foo.bar@domain.com>');

      await collector;

      expect(process.stdout.write).toBeCalledWithArgumentContaining(
        'Author must be set in the format',
      );
    });
  });

  describe('projectDescription', () => {
    const prompt = [prompts.find(({ name }) => name === 'projectDescription')];

    it('defaults to empty string', async () => {
      const collector = collectAnswers(prompt);
      await settingOfDefaultValue;
      emitInput(collector.ui);

      const answers = await collector;

      expect(answers.projectDescription).toEqual('');
    });

    it('returns answer we put in', async () => {
      const collector = collectAnswers(prompt);
      await settingOfDefaultValue;
      emitInput(collector.ui, 'Custom Answer');

      const answers = await collector;

      expect(answers.projectDescription).toEqual('Custom Answer');
    });
  });

  describe('projectPort', () => {
    const prompt = [prompts.find(({ name }) => name === 'projectPort')];

    it('defaults to 9001', async () => {
      const collector = collectAnswers(prompt);
      await settingOfDefaultValue;
      emitInput(collector.ui);

      const answers = await collector;

      expect(answers.projectPort).toEqual(9001);
    });

    it('returns answer we put in', async () => {
      const collector = collectAnswers(prompt);
      await settingOfDefaultValue;
      emitInput(collector.ui, 3000);

      const answers = await collector;

      expect(answers.projectPort).toEqual(3000);
    });

    it('Gives an error if we try to pass something thats not a number', async () => {
      jest.spyOn(process.stdout, 'write');
      const collector = collectAnswers(prompt);
      await settingOfDefaultValue;
      emitInput(collector.ui, 'not a number');
      await settingOfDefaultValue;
      emitInput(collector.ui, 3000);

      await collector;

      expect(process.stdout.write).toBeCalledWithArgumentContaining(
        'Port has to be a number',
      );
    });
  });
});
